package net.sf.maventaglib;

import java.io.File;
import java.io.IOException;

import org.apache.maven.artifact.handler.ArtifactHandler;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.maven.project.MavenProjectHelper;
import org.codehaus.plexus.archiver.ArchiverException;
import org.codehaus.plexus.archiver.jar.JarArchiver;

/**
 * @goal taglibdocjar
 * @phase package
 * @execute goal="taglibdoc"
 */

public class TaglibdocJar
    extends AbstractMojo
{

    /**
     * TldDoc output dir.
     * @parameter expression="${project.reporting.outputDirectory}/tlddoc"
     */
    private String tldDocDir;

    /**
     * @parameter expression="${project.build.finalName}"
     */
    private String finalName;

    /**
     * @parameter expression="${project}"
     * @readonly
     * @required
     */
    private MavenProject project;

    /**
     * @parameter expression="${component.org.apache.maven.project.MavenProjectHelper}
     */
    private MavenProjectHelper projectHelper;

    /**
     * @parameter expression="${attach}" default-value="true"
     */
    private boolean attach = true;

    public void execute()
        throws MojoExecutionException
    {
        ArtifactHandler artifactHandler = project.getArtifact().getArtifactHandler();
        if ( !"java".equals( artifactHandler.getLanguage() ) )
        {
            getLog().info( "Not executing tlddoc as the project is not a Java classpath-capable package" );
            return;
        }

        try
        {
            File outputFile = generateArchive( tldDocDir //+ "/tlddoc"
                                               , finalName + "-tlddoc.jar" );

            if ( !attach )
            {
                getLog().info( "NOT adding tlddoc to attached artifacts list." );
            }
            else
            {
                projectHelper.attachArtifact( project, "jar", "tlddoc", outputFile );
            }
        }
        catch ( ArchiverException e )
        {
            throw new MojoExecutionException( "Error while creating archive.", e );
        }
        catch ( IOException e )
        {
            throw new MojoExecutionException( "Error while creating archive.", e );
        }
    }

    private File generateArchive( String source, String target )
        throws MojoExecutionException, ArchiverException, IOException
    {
        File tlddocFiles = new File( source );

        if ( !tlddocFiles.exists() )
        {
            throw new MojoExecutionException( "tlddoc files not found." );
        }

        File tlddocJar = new File( tldDocDir, target );

        if ( tlddocJar.exists() )
        {
            tlddocJar.delete();
        }

        JarArchiver archiver = new JarArchiver();

        archiver.addDirectory( tlddocFiles );
        archiver.setDestFile( tlddocJar );
        archiver.createArchive();

        return tlddocJar;
    }

}
